part of '../../fling_units.dart';

/// Interprets [Frequency]ies as a specific unit.
class FrequencyInterpreter extends MeasurementInterpreter<Frequency> {
  @override
  Frequency call(num value, {Precision precision = Precision.max}) =>
      Frequency(_from(value), this, precision);

  /// Constructs a [FrequencyInterpreter].
  const FrequencyInterpreter._(
    String name,
    double multiplier, [
    MeasurementPrefix prefix = const MeasurementPrefix.unit(),
  ]) : super._(name, multiplier, prefix);

  /// Produces an equivalent [FrequencyInterpreter] but with an added multiplier prefix.
  FrequencyInterpreter _withPrefix(MeasurementPrefix prefix) =>
      FrequencyInterpreter._(_name, _unitMultiplier, prefix);

  /// The interpreter for hertz.
  static const _hertz = FrequencyInterpreter._('Hz', 1e0);
}

/// The interpreter for hertz.
const hertz = FrequencyInterpreter._hertz;

/// Applies a prefix to the frequency units.
mixin FrequencyPrefix {
  /// The prefix multiplier applied to this measurement.
  MeasurementPrefix get _prefix;

  /// Applies this to a hertz amount.
  FrequencyInterpreter get hertz =>
      FrequencyInterpreter._hertz._withPrefix(_prefix);
}

/// Represents a single dimension of frequency.
class Frequency extends Measurement<Frequency> {
  /// The SI unit associated with this measurement.
  static const siUnit = hertz;

  /// Zero frequency.
  const Frequency.zero([FrequencyInterpreter interpreter = siUnit])
      : super.zero(interpreter);

  /// Infinite frequency.
  const Frequency.infinite([FrequencyInterpreter interpreter = siUnit])
      : super.infinite(interpreter);

  /// Infinite negative frequency.
  const Frequency.negativeInfinite([FrequencyInterpreter interpreter = siUnit])
      : super.negativeInfinite(interpreter);

  /// Constructs a [Frequency] representing the sum of any number of other
  /// [Frequency]ies.
  Frequency.sum(Iterable<Frequency> parts,
      {final Precision precision = Precision.max})
      : super.sum(
          parts,
          precision,
        );

  /// Interprets this using the specified units.
  double as(MeasurementInterpreter<Frequency> interpreter) =>
      _preciseOf(interpreter);

  @override
  void acceptVisitor(MeasurementVisitor visitor) =>
      visitor.visitFrequency(this);

  @override
  Frequency _construct(
    double si,
    Precision precision, [
    MeasurementInterpreter<Frequency>? interpreter,
  ]) =>
      Frequency(si, interpreter ?? siUnit, precision);

  /// Constructs a [Frequency].
  const Frequency(
    double amount,
    MeasurementInterpreter<Frequency> interpreter, [
    Precision precision = Precision.max,
  ]) : super(amount, precision, interpreter);
}
